@@ -1,8 +1,9 @@
 use crate::{
-    completions::{complete_profile, complete_recipe},
     GlobalArgs, Subcommand,
+    completions::{complete_profile, complete_recipe},
 };
-use anyhow::{anyhow, Context};
+use anyhow::{Context, anyhow};
+use async_trait::async_trait;
 use clap::{Parser, ValueHint};
 use clap_complete::ArgValueCompleter;
 use dialoguer::{Input, Password, Select as DialoguerSelect};
@@ -10,15 +11,19 @@ use indexmap::IndexMap;
 use itertools::Itertools;
 use slumber_config::Config;
 use slumber_core::{
-    collection::{Collection, CollectionFile, ProfileId, RecipeId},
-    db::{CollectionDatabase, Database, DatabaseMode},
+    collection::{ProfileId, RecipeId},
+    database::{CollectionDatabase, Database},
     http::{
-        BuildOptions, HttpEngine, RequestRecord, RequestSeed, RequestTicket,
+        BuildOptions, Exchange, HttpEngine, RequestRecord, RequestSeed,
         ResponseRecord,
     },
-    template::{Prompt, Prompter, Select, TemplateContext, TemplateError},
-    util::{MaybeStr, ResultTraced},
+    template::{
+        HttpProvider, Prompt, Prompter, Select, TemplateContext,
+        TriggeredRequestError,
+    },
+    util::MaybeStr,
 };
+use slumber_util::ResultTraced;
 use std::{
     error::Error,
     fs::OpenOptions,
@@ -44,7 +49,7 @@ pub struct RequestCommand {
     display: DisplayExchangeCommand,
 
     /// Just print the generated request, instead of sending it. Triggered
-    /// sub-requests will also not be executed.
+    /// sub-requests will also not be executed. Implies `--verbose`.
     #[clap(long)]
     dry_run: bool,
 
@@ -52,6 +57,17 @@ pub struct RequestCommand {
     /// <400, exit code is 0. If it's >=400, exit code is 2.
     #[clap(long)]
     exit_status: bool,
+
+    /// Persist the completed request to Slumber's history database. By
+    /// default, CLI-based requests are not persisted. The CLI ignores the
+    /// `persist` field in the global configuration and recipe definition; this
+    /// flag is the only thing that controls persistence for the CLI.
+    ///
+    /// If the request triggers any upstream chained requests while building,
+    /// those requests will NOT be persisted, regardless of the presence of
+    /// this flag.
+    #[clap(long)]
+    persist: bool,
 }
 
 /// A helper for any subcommand that needs to build requests. This handles
@@ -101,32 +117,42 @@ pub struct DisplayExchangeCommand {
 }
 
 impl Subcommand for RequestCommand {
-    async fn execute(self, global: GlobalArgs) -> anyhow::Result<ExitCode> {
-        let (database, ticket) = self
+    async fn execute(mut self, global: GlobalArgs) -> anyhow::Result<ExitCode> {
+        // Don't execute sub-requests in a dry run
+        let trigger_dependencies = !self.dry_run;
+        let (database, http_engine, seed, template_context) = self
             .build_request
-            // Don't execute sub-requests in a dry run
-            .build_request(global, !self.dry_run)
-            .await
-            .map_err(|error| {
+            .build_seed(global, trigger_dependencies)?;
+        let ticket = http_engine.build(seed, &template_context).await.map_err(
+            |error| {
                 // If the build failed because triggered requests are disabled,
                 // replace it with a custom error message
-                if TemplateError::has_trigger_disabled_error(&error) {
-                    error.context(
+                if error.has_trigger_disabled_error() {
+                    error.source.context(
                         "Triggered requests are disabled with `--dry-run`",
                     )
                 } else {
-                    error
+                    error.source
                 }
-            })?;
+            },
+        )?;
 
         if self.dry_run {
-            println!("{:#?}", ticket.record());
+            // With --dry-run, we don't do anything unless the verbose flag is
+            // enabled. Exiting with no output is confusing so for it enabled
+            // here.
+            self.display.verbose = true;
+            self.display.write_request(ticket.record());
             Ok(ExitCode::SUCCESS)
         } else {
             self.display.write_request(ticket.record());
 
             // Run the request
-            let exchange = ticket.send(&database).await?;
+            let exchange = ticket.send().await?;
+            if self.persist {
+                // Error here shouldn't be propagated, just logged
+                let _ = database.insert_exchange(&exchange).traced();
+            }
             let status = exchange.response.status;
 
             self.display.write_response(&exchange.response)?;
@@ -141,25 +167,32 @@ impl Subcommand for RequestCommand {
 }
 
 impl BuildRequestCommand {
-    /// Render the request specified by the user. This returns the HTTP engine
-    /// too so it can be re-used if necessary (iff `trigger_dependencies` is
-    /// enabled).
+    /// Get all the components needed to build a request for the recipe selected
+    /// by this command. The returned values can be used to build the request
+    /// in whatever format the consumer needs.
     ///
-    /// `trigger_dependencies` controls whether chained requests can be executed
-    /// if their triggers apply.
-    pub async fn build_request(
+    /// ## Parameters
+    ///
+    /// - `global`: Global arguments for the CLI
+    /// - `persist`: Will the request/response be persisted in history? This
+    ///   needs to be passed here (at build time) because it's baked into the
+    ///   HTTP engine's config.
+    /// - `trigger_dependencies`: Whether chained requests can be executed if
+    ///   their triggers apply
+    pub fn build_seed(
         self,
         global: GlobalArgs,
         trigger_dependencies: bool,
-    ) -> anyhow::Result<(CollectionDatabase, RequestTicket)> {
-        let collection_path = CollectionFile::try_path(None, global.file)?;
+    ) -> anyhow::Result<(
+        CollectionDatabase,
+        HttpEngine,
+        RequestSeed,
+        TemplateContext,
+    )> {
+        let collection_file = global.collection_file()?;
         let config = Config::load()?;
-        let collection = Collection::load(&collection_path)?;
-        // Open DB in readonly. Storing requests in history from the CLI isn't
-        // really intuitive, and could have a large perf impact for scripting
-        // and large responses
-        let database = Database::load()?
-            .into_collection(&collection_path, DatabaseMode::ReadOnly)?;
+        let collection = collection_file.load()?;
+        let database = Database::load()?.into_collection(&collection_file)?;
         let http_engine = HttpEngine::new(&config.http);
 
         // Validate profile ID, so we can provide a good error if it's invalid
@@ -183,21 +216,17 @@ impl BuildRequestCommand {
         let template_context = TemplateContext {
             selected_profile,
             collection: collection.into(),
-            // Passing the HTTP engine is how we tell the template renderer that
-            // it's ok to execute subrequests during render
-            http_engine: if trigger_dependencies {
-                Some(http_engine.clone())
-            } else {
-                None
-            },
-            database: database.clone(),
+            http_provider: Box::new(CliHttpProvider {
+                database: database.clone(),
+                http_engine: http_engine.clone(),
+                trigger_dependencies,
+            }),
             overrides,
             prompter: Box::new(CliPrompter),
             state: Default::default(),
         };
         let seed = RequestSeed::new(self.recipe_id, BuildOptions::default());
-        let request = http_engine.build(seed, &template_context).await?;
-        Ok((database, request))
+        Ok((database, http_engine, seed, template_context))
     }
 }
 
@@ -213,6 +242,10 @@ impl DisplayExchangeCommand {
             for (header, value) in &request.headers {
                 eprintln!("> {}: {}", header, MaybeStr(value.as_bytes()));
             }
+            if let Some(body) = &request.body {
+                let text = std::str::from_utf8(body).unwrap_or("<binary>");
+                eprintln!("> {text}");
+            }
         }
     }
 
@@ -271,6 +304,41 @@ impl DisplayExchangeCommand {
     }
 }
 
+/// [HttpProvider] for the CLI. This will _not_ performance any persistence;
+/// that should be handled by the request command implementation as needed.
+#[derive(Debug)]
+struct CliHttpProvider {
+    database: CollectionDatabase,
+    http_engine: HttpEngine,
+    trigger_dependencies: bool,
+}
+
+#[async_trait]
+impl HttpProvider for CliHttpProvider {
+    async fn get_latest_request(
+        &self,
+        profile_id: Option<&ProfileId>,
+        recipe_id: &RecipeId,
+    ) -> anyhow::Result<Option<Exchange>> {
+        self.database
+            .get_latest_request(profile_id.into(), recipe_id)
+    }
+
+    async fn send_request(
+        &self,
+        seed: RequestSeed,
+        template_context: &TemplateContext,
+    ) -> Result<Exchange, TriggeredRequestError> {
+        if self.trigger_dependencies {
+            let ticket = self.http_engine.build(seed, template_context).await?;
+            let exchange = ticket.send().await?;
+            Ok(exchange)
+        } else {
+            Err(TriggeredRequestError::NotAllowed)
+        }
+    }
+}
+
 /// Prompt the user for input on the CLI
 #[derive(Debug)]
 struct CliPrompter;
