@@ -2,8 +2,8 @@
 
 use crate::{
     collection::{
-        recipe_tree::RecipeNode, Chain, ChainId, Profile, ProfileId, Recipe,
-        RecipeBody, RecipeId,
+        Chain, ChainId, Profile, ProfileId, Recipe, RecipeBody, RecipeId,
+        recipe_tree::RecipeNode,
     },
     http::content_type::ContentType,
     template::Template,
@@ -12,12 +12,9 @@ use anyhow::Context;
 use indexmap::IndexMap;
 use itertools::Itertools;
 use serde::{
-    de::{
-        self, EnumAccess, Error as _, MapAccess, SeqAccess, VariantAccess,
-        Visitor,
-    },
-    ser::Error as _,
     Deserialize, Deserializer, Serialize, Serializer,
+    de::{self, EnumAccess, Error as _, VariantAccess, Visitor},
+    ser::Error as _,
 };
 use std::{hash::Hash, str::FromStr};
 
@@ -85,6 +82,12 @@ impl HasId for Chain {
     }
 }
 
+/// Default value generator for `Recipe::persist`. All recipes are persisted by
+/// default
+pub fn persist_default() -> bool {
+    true
+}
+
 /// Deserialize a map, and update each key so its `id` field matches its key in
 /// the map. Useful if you need to access the ID when you only have a value
 /// available, not the full entry.
@@ -138,8 +141,12 @@ where
 /// of `key: value`. Serialie back to a sequence `key=value`, since that will
 /// always support duplicate keys
 pub mod serde_query_parameters {
-    use super::*;
-    use serde::ser::SerializeSeq;
+    use crate::template::Template;
+    use serde::{
+        Deserializer, Serializer,
+        de::{self, MapAccess, SeqAccess, Visitor},
+        ser::SerializeSeq,
+    };
 
     pub fn serialize<S>(
         query_parameters: &Vec<(String, Template)>,
@@ -403,10 +410,10 @@ impl<'de> Deserialize<'de> for RecipeBody {
 pub mod serde_duration {
     use derive_more::Display;
     use itertools::Itertools;
-    use serde::{de::Error, Deserialize, Deserializer, Serializer};
+    use serde::{Deserialize, Deserializer, Serializer, de::Error};
     use std::time::Duration;
     use strum::{EnumIter, EnumString, IntoEnumIterator};
-    use winnow::{ascii::digit1, token::take_while, PResult, Parser};
+    use winnow::{ModalResult, Parser, ascii::digit1, token::take_while};
 
     #[derive(Debug, Display, EnumIter, EnumString)]
     enum Unit {
@@ -440,11 +447,11 @@ pub mod serde_duration {
     where
         D: Deserializer<'de>,
     {
-        fn quantity(input: &mut &str) -> PResult<u64> {
+        fn quantity(input: &mut &str) -> ModalResult<u64> {
             digit1.parse_to().parse_next(input)
         }
 
-        fn unit<'a>(input: &mut &'a str) -> PResult<&'a str> {
+        fn unit<'a>(input: &mut &'a str) -> ModalResult<&'a str> {
             take_while(1.., char::is_alphabetic).parse_next(input)
         }
 
@@ -479,18 +486,18 @@ pub mod serde_duration {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::assert_err;
     use indexmap::indexmap;
     use rstest::rstest;
     use serde::Serialize;
     use serde_json::json;
     use serde_test::{
-        assert_de_tokens, assert_de_tokens_error, assert_ser_tokens, Token,
+        Token, assert_de_tokens, assert_de_tokens_error, assert_ser_tokens,
     };
     use serde_yaml::{
-        value::{Tag, TaggedValue},
         Mapping,
+        value::{Tag, TaggedValue},
     };
+    use slumber_util::assert_err;
     use std::time::Duration;
 
     #[rstest]
@@ -515,7 +522,7 @@ mod tests {
         #[derive(Debug, Deserialize)]
         #[serde(transparent)]
         struct Wrap(
-            #[allow(dead_code)]
+            #[expect(dead_code)]
             #[serde(deserialize_with = "deserialize_profiles")]
             IndexMap<ProfileId, Profile>,
         );
@@ -708,12 +715,12 @@ mod tests {
     #[case::seconds_longer("100s", Duration::from_secs(100))]
     #[case::minutes("3m", Duration::from_secs(180))]
     #[case::hours("3h", Duration::from_secs(10800))]
-    #[case::days("2d", Duration::from_secs(172800))]
+    #[case::days("2d", Duration::from_secs(172_800))]
     fn test_deserialize_duration(
         #[case] s: &'static str,
         #[case] expected: Duration,
     ) {
-        assert_de_tokens(&WrapDuration(expected), &[Token::Str(s)])
+        assert_de_tokens(&WrapDuration(expected), &[Token::Str(s)]);
     }
 
     #[rstest]
@@ -741,7 +748,7 @@ mod tests {
         #[case] s: &'static str,
         #[case] error: &str,
     ) {
-        assert_de_tokens_error::<WrapDuration>(&[Token::Str(s)], error)
+        assert_de_tokens_error::<WrapDuration>(&[Token::Str(s)], error);
     }
 
     /// Build a YAML mapping
