@@ -35,13 +35,6 @@ pub struct Collection {
     /// intuitive
     #[serde(default, rename = "requests")]
     pub recipes: RecipeTree,
-    /// A hack-ish to allow users to add arbitrary data to their collection
-    /// file without triggering a unknown field error. Ideally we could
-    /// ignore anything that starts with `.` (recursively) but that
-    /// requires a custom serde impl for each type, or changes to the macro
-    #[serde(default, skip_serializing, rename = ".ignore")]
-    #[expect(clippy::pub_underscore_fields)]
-    pub _ignore: serde::de::IgnoredAny,
 }
 
 impl Collection {
@@ -181,17 +174,28 @@ pub struct Recipe {
     pub id: RecipeId,
     #[serde(default = "cereal::persist_default")]
     pub persist: bool,
+    #[serde(skip_serializing_if = "Option::is_none")]
     pub name: Option<String>,
     /// *Not* a template string because the usefulness doesn't justify the
     /// complexity. This gives the user an immediate error if the method is
     /// wrong which is helpful.
     pub method: HttpMethod,
     pub url: Template,
+    #[serde(skip_serializing_if = "Option::is_none")]
     pub body: Option<RecipeBody>,
+    #[serde(skip_serializing_if = "Option::is_none")]
     pub authentication: Option<Authentication>,
-    #[serde(default, with = "cereal::serde_query_parameters")]
+    #[serde(
+        default,
+        with = "cereal::serde_query_parameters",
+        skip_serializing_if = "Vec::is_empty"
+    )]
     pub query: Vec<(String, Template)>,
-    #[serde(default, deserialize_with = "cereal::deserialize_headers")]
+    #[serde(
+        default,
+        deserialize_with = "cereal::deserialize_headers",
+        skip_serializing_if = "IndexMap::is_empty"
+    )]
     pub headers: IndexMap<String, Template>,
 }
 
