@@ -6,7 +6,7 @@
 use crate::{
     collection::{Authentication, ProfileId, RecipeBody, RecipeId},
     http::content_type::ContentType,
-    template::Template,
+    template::{ChainError, Template, TemplateError, TriggeredRequestError},
 };
 use anyhow::Context;
 use bytes::Bytes;
@@ -15,15 +15,11 @@ use derive_more::{Display, From, FromStr};
 use itertools::Itertools;
 use mime::Mime;
 use reqwest::{
-    header::{self, HeaderMap},
     Body, Client, Request, StatusCode, Url,
+    header::{self, HeaderMap},
 };
 use serde::{Deserialize, Serialize};
-use std::{
-    collections::HashMap,
-    fmt::{Debug, Write},
-    sync::Arc,
-};
+use std::{collections::HashMap, fmt::Debug, sync::Arc};
 use strum::{EnumIter, IntoEnumIterator};
 use thiserror::Error;
 use tracing::error;
@@ -39,11 +35,12 @@ use uuid::Uuid;
     Eq,
     FromStr,
     Hash,
+    Ord,
     PartialEq,
+    PartialOrd,
     Serialize,
     Deserialize,
 )]
-#[cfg_attr(test, derive(PartialOrd, Ord))]
 pub struct RequestId(pub Uuid);
 
 impl RequestId {
@@ -140,7 +137,7 @@ impl TryFrom<String> for HttpVersion {
 #[derive(Debug, Error)]
 #[error(
     "Invalid HTTP version `{input}`. Must be one of: {}",
-    HttpVersion::iter().map(|method| method.to_str()).format(", "),
+    HttpVersion::iter().map(HttpVersion::to_str).format(", "),
 )]
 pub struct HttpVersionParseError {
     input: String,
@@ -236,7 +233,7 @@ impl TryFrom<String> for HttpMethod {
 #[derive(Debug, Error)]
 #[error(
     "Invalid HTTP method `{input}`. Must be one of: {}",
-    HttpMethod::iter().map(|method| method.to_str()).format(", "),
+    HttpMethod::iter().map(HttpMethod::to_str).format(", "),
 )]
 pub struct HttpMethodParseError {
     input: String,
@@ -358,8 +355,9 @@ impl RequestTicket {
 
 /// A complete request+response pairing. This is generated by
 /// [RequestTicket::send] when a response is received successfully for a sent
-/// request.
-#[derive(Debug)]
+/// request. This is cheaply cloneable because the request and response are
+/// both wrapped in `Arc`.
+#[derive(Clone, Debug)]
 #[cfg_attr(any(test, feature = "test"), derive(PartialEq))]
 pub struct Exchange {
     /// ID to uniquely refer to this exchange
@@ -379,31 +377,31 @@ impl Exchange {
     pub fn duration(&self) -> Duration {
         self.end_time - self.start_time
     }
+
+    pub fn summary(&self) -> ExchangeSummary {
+        ExchangeSummary {
+            id: self.id,
+            recipe_id: self.request.recipe_id.clone(),
+            profile_id: self.request.profile_id.clone(),
+            start_time: self.start_time,
+            end_time: self.end_time,
+            status: self.response.status,
+        }
+    }
 }
 
 /// Metadata about an exchange. Useful in lists where request/response content
 /// isn't needed.
-#[derive(Clone, Debug)]
+#[derive(Clone, Debug, PartialEq)]
 pub struct ExchangeSummary {
     pub id: RequestId,
+    pub recipe_id: RecipeId,
     pub profile_id: Option<ProfileId>,
     pub start_time: DateTime<Utc>,
     pub end_time: DateTime<Utc>,
     pub status: StatusCode,
 }
 
-impl From<&Exchange> for ExchangeSummary {
-    fn from(exchange: &Exchange) -> Self {
-        Self {
-            id: exchange.id,
-            profile_id: exchange.request.profile_id.clone(),
-            start_time: exchange.start_time,
-            end_time: exchange.end_time,
-            status: exchange.response.status,
-        }
-    }
-}
-
 /// Data for an HTTP request. This is similar to [reqwest::Request], but differs
 /// in some key ways:
 /// - Each [reqwest::Request] can only exist once (from creation to sending),
@@ -479,32 +477,6 @@ impl RequestRecord {
         content_type_header(&self.headers)
     }
 
-    /// Generate a cURL command equivalent to this request
-    ///
-    /// This only fails if one of the headers or body is binary and can't be
-    /// converted to UTF-8.
-    pub fn to_curl(&self) -> anyhow::Result<String> {
-        let mut buf = String::new();
-
-        // These writes are all infallible because we're writing to a string,
-        // but use ? because it's shorter than unwrap().
-        let method = &self.method;
-        let url = &self.url;
-        write!(&mut buf, "curl -X{method} --url '{url}'")?;
-
-        for (header, value) in &self.headers {
-            let value =
-                value.to_str().context("Error decoding header value")?;
-            write!(&mut buf, " --header '{header}: {value}'")?;
-        }
-
-        if let Some(body) = &self.body_str()? {
-            write!(&mut buf, " --data '{body}'")?;
-        }
-
-        Ok(buf)
-    }
-
     pub fn body(&self) -> Option<&[u8]> {
         self.body.as_deref()
     }
@@ -523,14 +495,14 @@ impl RequestRecord {
 }
 
 #[cfg(any(test, feature = "test"))]
-impl crate::test_util::Factory for RequestRecord {
-    fn factory(_: ()) -> Self {
+impl slumber_util::Factory for RequestRecord {
+    fn factory((): ()) -> Self {
         Self::factory(RequestId::new())
     }
 }
 
 #[cfg(any(test, feature = "test"))]
-impl crate::test_util::Factory<RequestId> for RequestRecord {
+impl slumber_util::Factory<RequestId> for RequestRecord {
     fn factory(id: RequestId) -> Self {
         Self {
             id,
@@ -541,9 +513,7 @@ impl crate::test_util::Factory<RequestId> for RequestRecord {
 
 /// Customize profile and recipe ID
 #[cfg(any(test, feature = "test"))]
-impl crate::test_util::Factory<(Option<ProfileId>, RecipeId)>
-    for RequestRecord
-{
+impl slumber_util::Factory<(Option<ProfileId>, RecipeId)> for RequestRecord {
     fn factory((profile_id, recipe_id): (Option<ProfileId>, RecipeId)) -> Self {
         use crate::test_util::header_map;
         Self {
@@ -564,14 +534,14 @@ impl crate::test_util::Factory<(Option<ProfileId>, RecipeId)>
 }
 
 #[cfg(any(test, feature = "test"))]
-impl crate::test_util::Factory for ResponseRecord {
-    fn factory(_: ()) -> Self {
+impl slumber_util::Factory for ResponseRecord {
+    fn factory((): ()) -> Self {
         Self::factory(RequestId::new())
     }
 }
 
 #[cfg(any(test, feature = "test"))]
-impl crate::test_util::Factory<RequestId> for ResponseRecord {
+impl slumber_util::Factory<RequestId> for ResponseRecord {
     fn factory(id: RequestId) -> Self {
         Self {
             id,
@@ -583,7 +553,7 @@ impl crate::test_util::Factory<RequestId> for ResponseRecord {
 }
 
 #[cfg(any(test, feature = "test"))]
-impl crate::test_util::Factory<StatusCode> for ResponseRecord {
+impl slumber_util::Factory<StatusCode> for ResponseRecord {
     fn factory(status: StatusCode) -> Self {
         Self {
             id: RequestId::new(),
@@ -595,23 +565,41 @@ impl crate::test_util::Factory<StatusCode> for ResponseRecord {
 }
 
 #[cfg(any(test, feature = "test"))]
-impl crate::test_util::Factory for Exchange {
-    fn factory(_: ()) -> Self {
+impl slumber_util::Factory for Exchange {
+    fn factory((): ()) -> Self {
         Self::factory((None, RecipeId::factory(())))
     }
 }
 
 /// Customize recipe ID
 #[cfg(any(test, feature = "test"))]
-impl crate::test_util::Factory<RecipeId> for Exchange {
+impl slumber_util::Factory<RecipeId> for Exchange {
     fn factory(params: RecipeId) -> Self {
         Self::factory((None, params))
     }
 }
 
+/// Customize request, profile, and recipe ID
+#[cfg(any(test, feature = "test"))]
+impl slumber_util::Factory<(RequestId, Option<ProfileId>, RecipeId)>
+    for Exchange
+{
+    fn factory(
+        (id, profile_id, recipe_id): (RequestId, Option<ProfileId>, RecipeId),
+    ) -> Self {
+        Self::factory((
+            RequestRecord {
+                id,
+                ..RequestRecord::factory((profile_id, recipe_id))
+            },
+            ResponseRecord::factory(id),
+        ))
+    }
+}
+
 /// Customize profile and recipe ID
 #[cfg(any(test, feature = "test"))]
-impl crate::test_util::Factory<(Option<ProfileId>, RecipeId)> for Exchange {
+impl slumber_util::Factory<(Option<ProfileId>, RecipeId)> for Exchange {
     fn factory(params: (Option<ProfileId>, RecipeId)) -> Self {
         let id = RequestId::new();
         Self::factory((
@@ -626,7 +614,7 @@ impl crate::test_util::Factory<(Option<ProfileId>, RecipeId)> for Exchange {
 
 /// Custom request and response
 #[cfg(any(test, feature = "test"))]
-impl crate::test_util::Factory<(RequestRecord, ResponseRecord)> for Exchange {
+impl slumber_util::Factory<(RequestRecord, ResponseRecord)> for Exchange {
     fn factory((request, response): (RequestRecord, ResponseRecord)) -> Self {
         // Request and response should've been generated from the same ID,
         // otherwise we're going to see some shitty bugs
@@ -645,7 +633,7 @@ impl crate::test_util::Factory<(RequestRecord, ResponseRecord)> for Exchange {
 }
 
 #[cfg(any(test, feature = "test"))]
-impl crate::test_util::Factory<RequestId> for Exchange {
+impl slumber_util::Factory<RequestId> for Exchange {
     fn factory(id: RequestId) -> Self {
         Self::factory((RequestRecord::factory(id), ResponseRecord::factory(id)))
     }
@@ -805,7 +793,7 @@ pub struct RequestBuildError {
     /// There are multiple possible error types and anyhow's Error makes
     /// display easier
     #[source]
-    pub error: anyhow::Error,
+    pub source: anyhow::Error,
 
     /// ID of the profile being rendered under
     pub profile_id: Option<ProfileId>,
@@ -819,6 +807,26 @@ pub struct RequestBuildError {
     pub end_time: DateTime<Utc>,
 }
 
+impl RequestBuildError {
+    /// Does this error have *any* error in its chain that contains
+    /// [TriggeredRequestError::NotAllowed]? This makes it easy to attach
+    /// additional error context.
+    pub fn has_trigger_disabled_error(&self) -> bool {
+        self.source.chain().any(|error| {
+            matches!(
+                error.downcast_ref(),
+                Some(TemplateError::Chain {
+                    error: ChainError::Trigger {
+                        error: TriggeredRequestError::NotAllowed,
+                        ..
+                    },
+                    ..
+                })
+            )
+        })
+    }
+}
+
 #[cfg(any(test, feature = "test"))]
 impl PartialEq for RequestBuildError {
     fn eq(&self, other: &Self) -> bool {
@@ -827,7 +835,7 @@ impl PartialEq for RequestBuildError {
             && self.id == other.id
             && self.start_time == other.start_time
             && self.end_time == other.end_time
-            && self.error.to_string() == other.error.to_string()
+            && self.source.to_string() == other.source.to_string()
     }
 }
 
@@ -866,10 +874,10 @@ impl PartialEq for RequestError {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::test_util::{header_map, Factory};
+    use crate::test_util::header_map;
     use indexmap::indexmap;
     use rstest::rstest;
-    use serde_json::json;
+    use slumber_util::Factory;
 
     #[rstest]
     #[case::content_disposition(
@@ -909,27 +917,4 @@ mod tests {
     ) {
         assert_eq!(response.file_name().as_deref(), expected);
     }
-
-    #[test]
-    fn test_to_curl() {
-        let headers = indexmap! {
-            "accept" => "application/json",
-            "content-type" => "application/json",
-        };
-        let body = json!({"data": "value"});
-        let request = RequestRecord {
-            method: HttpMethod::Delete,
-            headers: header_map(headers),
-            body: Some(serde_json::to_vec(&body).unwrap().into()),
-            ..RequestRecord::factory(())
-        };
-
-        assert_eq!(
-            request.to_curl().unwrap(),
-            "curl -XDELETE --url 'http://localhost/url' \
-            --header 'accept: application/json' \
-            --header 'content-type: application/json' \
-            --data '{\"data\":\"value\"}'"
-        );
-    }
 }
