@@ -52,7 +52,6 @@ fn template_context(
             recipes: by_id(recipes).into(),
             profiles: by_id([profile]),
             chains: by_id(chains),
-            ..Collection::factory(())
         }
         .into(),
         selected_profile: Some(profile_id.clone()),
@@ -68,7 +67,17 @@ async fn mock_server() -> String {
     let host = server.uri();
     Mock::given(matchers::method("GET"))
         .and(matchers::path("/get"))
-        .respond_with(ResponseTemplate::new(200).set_body_string("hello!"))
+        .respond_with(
+            ResponseTemplate::new(StatusCode::OK).set_body_string("hello!"),
+        )
+        .mount(&server)
+        .await;
+    Mock::given(matchers::method("GET"))
+        .and(matchers::path("/redirect"))
+        .respond_with(
+            ResponseTemplate::new(StatusCode::MOVED_PERMANENTLY)
+                .insert_header("Location", format!("{host}/get")),
+        )
         .mount(&server)
         .await;
     host
@@ -82,7 +91,7 @@ async fn mock_server() -> String {
 #[case::safe("safe", false)]
 #[case::danger("danger", true)]
 fn test_get_client(
-    http_engine: &HttpEngine,
+    http_engine: HttpEngine,
     #[case] hostname: &str,
     #[case] expected_danger: bool,
 ) {
@@ -100,7 +109,7 @@ fn test_get_client(
 
 #[rstest]
 #[tokio::test]
-async fn test_build_request(http_engine: &HttpEngine) {
+async fn test_build_request(http_engine: HttpEngine) {
     let recipe = Recipe {
         method: HttpMethod::Post,
         url: "{{host}}/users/{{user_id}}".into(),
@@ -163,7 +172,7 @@ async fn test_build_request(http_engine: &HttpEngine) {
 /// should *not* be built
 #[rstest]
 #[tokio::test]
-async fn test_build_url(http_engine: &HttpEngine) {
+async fn test_build_url(http_engine: HttpEngine) {
     let recipe = Recipe {
         url: "{{host}}/users/{{user_id}}".into(),
         query: vec![
@@ -214,7 +223,7 @@ async fn test_build_url(http_engine: &HttpEngine) {
 )]
 #[tokio::test]
 async fn test_build_body(
-    http_engine: &HttpEngine,
+    http_engine: HttpEngine,
     invalid_utf8_chain: ChainSource,
     #[case] body: RecipeBody,
     #[case] expected_body: &[u8],
@@ -262,7 +271,7 @@ async fn test_build_body(
 #[case::bearer(Authentication::Bearer("{{token}}".into()), "Bearer tokenzzz")]
 #[tokio::test]
 async fn test_authentication(
-    http_engine: &HttpEngine,
+    http_engine: HttpEngine,
     #[case] authentication: Authentication,
     #[case] expected_header: &str,
 ) {
@@ -361,7 +370,7 @@ async fn test_authentication(
 )]
 #[tokio::test]
 async fn test_structured_body(
-    http_engine: &HttpEngine,
+    http_engine: HttpEngine,
     invalid_utf8_chain: ChainSource,
     #[case] body: RecipeBody,
     #[case] content_type: Option<&str>,
@@ -438,7 +447,7 @@ async fn test_structured_body(
 /// bodies
 #[rstest]
 #[tokio::test]
-async fn test_build_options(http_engine: &HttpEngine) {
+async fn test_build_options(http_engine: HttpEngine) {
     let recipe = Recipe {
         authentication: Some(Authentication::Basic {
             username: "username".into(),
@@ -523,7 +532,7 @@ async fn test_build_options(http_engine: &HttpEngine) {
 /// because it's incompatible with testing raw body overrides
 #[rstest]
 #[tokio::test]
-async fn test_build_options_form(http_engine: &HttpEngine) {
+async fn test_build_options_form(http_engine: HttpEngine) {
     let recipe = Recipe {
         // This should implicitly set the content-type header
         body: Some(RecipeBody::FormUrlencoded(indexmap! {
@@ -575,7 +584,7 @@ async fn test_build_options_form(http_engine: &HttpEngine) {
 /// so that the chain is only computed once
 #[rstest]
 #[tokio::test]
-async fn test_chain_duplicate(http_engine: &HttpEngine) {
+async fn test_chain_duplicate(http_engine: HttpEngine) {
     let recipe = Recipe {
         method: HttpMethod::Post,
         url: "{{host}}/{{chains.text}}".into(),
@@ -602,7 +611,7 @@ async fn test_chain_duplicate(http_engine: &HttpEngine) {
 /// Test launching a built request
 #[rstest]
 #[tokio::test]
-async fn test_send_request(http_engine: &HttpEngine) {
+async fn test_send_request(http_engine: HttpEngine) {
     let host = mock_server().await;
     let recipe = Recipe {
         url: format!("{host}/get").as_str().into(),
@@ -684,7 +693,7 @@ fn test_trim_bytes(#[case] bytes: &[u8], #[case] expected: &[u8]) {
 /// Build a curl command with query parameters and headers
 #[rstest]
 #[tokio::test]
-async fn test_build_curl(http_engine: &HttpEngine) {
+async fn test_build_curl(http_engine: HttpEngine) {
     let recipe = Recipe {
         method: HttpMethod::Get,
         query: vec![
@@ -735,7 +744,7 @@ async fn test_build_curl(http_engine: &HttpEngine) {
 )]
 #[tokio::test]
 async fn test_build_curl_authentication(
-    http_engine: &HttpEngine,
+    http_engine: HttpEngine,
     #[case] authentication: Authentication,
     #[case] expected_arguments: &str,
 ) {
@@ -783,7 +792,7 @@ async fn test_build_curl_authentication(
 )]
 #[tokio::test]
 async fn test_build_curl_body(
-    http_engine: &HttpEngine,
+    http_engine: HttpEngine,
     #[case] body: RecipeBody,
     #[case] expected_arguments: &str,
 ) {
@@ -803,3 +812,48 @@ async fn test_build_curl_body(
         format!("curl -XGET --url 'http://localhost/url' {expected_arguments}");
     assert_eq!(command, expected_command);
 }
+
+/// By default, the engine will follow 3xx redirects
+#[rstest]
+#[tokio::test]
+async fn test_follow_redirects(http_engine: HttpEngine) {
+    let host = mock_server().await;
+    let recipe = Recipe {
+        url: format!("{host}/redirect").as_str().into(),
+        ..Recipe::factory(())
+    };
+    let recipe_id = recipe.id.clone();
+    let template_context = template_context([recipe], []);
+
+    // Build+send the request
+    let seed = RequestSeed::new(recipe_id, BuildOptions::default());
+    let ticket = http_engine.build(seed, &template_context).await.unwrap();
+    let exchange = ticket.send().await.unwrap();
+
+    // Should hit /redirect which redirects to /get, which returns the body
+    assert_eq!(exchange.response.status, StatusCode::OK);
+    assert_eq!(exchange.response.body.bytes().as_ref(), b"hello!");
+}
+
+/// Client should not follow redirects when the config field is disabled
+#[tokio::test]
+async fn test_follow_redirects_disabled() {
+    let http_engine = HttpEngine::new(&HttpEngineConfig {
+        follow_redirects: false,
+        ..Default::default()
+    });
+    let host = mock_server().await;
+    let recipe = Recipe {
+        url: format!("{host}/redirect").as_str().into(),
+        ..Recipe::factory(())
+    };
+    let recipe_id = recipe.id.clone();
+    let template_context = template_context([recipe], []);
+
+    // Build+send the request
+    let seed = RequestSeed::new(recipe_id, BuildOptions::default());
+    let ticket = http_engine.build(seed, &template_context).await.unwrap();
+    let exchange = ticket.send().await.unwrap();
+
+    assert_eq!(exchange.response.status, StatusCode::MOVED_PERMANENTLY);
+}
