@@ -1,6 +1,3 @@
-#![forbid(unsafe_code)]
-#![deny(clippy::all)]
-
 //! Core frontend-agnostic functionality for Slumber, agnostic of the front end.
 //!
 //! **This crate is not semver compliant**. The version is locked to the root
@@ -8,7 +5,7 @@
 //! do so at your own risk of breakage.
 
 pub mod collection;
-pub mod db;
+pub mod database;
 pub mod http;
 pub mod template;
 #[cfg(any(test, feature = "test"))]
