@@ -6,7 +6,7 @@ use crossterm::event::{
     MouseEvent, MouseEventKind,
 };
 use derive_more::Display;
-use indexmap::{indexmap, IndexMap};
+use indexmap::{IndexMap, indexmap};
 use slumber_config::{Action, InputBinding, KeyCombination};
 use tracing::trace;
 
@@ -59,7 +59,7 @@ impl InputEngine {
     /// alone.
     pub fn add_hint(&self, label: impl Display, action: Action) -> String {
         if let Some(binding) = self.binding(action) {
-            format!("{} ({})", label, binding)
+            format!("{label} ({binding})")
         } else {
             label.to_string()
         }
@@ -113,7 +113,7 @@ impl InputEngine {
     /// useless and noisy, or because they actually cause bugs (e.g. double key
     /// presses).
     pub fn event_to_message(&self, event: Event) -> Option<Message> {
-        if !matches!(
+        if matches!(
             event,
             Event::FocusGained
                 | Event::FocusLost
@@ -130,10 +130,10 @@ impl InputEngine {
                     ..
                 })
         ) {
+            None
+        } else {
             let action = self.action(&event);
             Some(Message::Input { event, action })
-        } else {
-            None
         }
     }
 }
@@ -176,6 +176,7 @@ impl Default for InputEngine {
                 Action::Submit => KeyCode::Enter.into(),
                 Action::Toggle => KeyCode::Char(' ').into(),
                 Action::Cancel => KeyCode::Esc.into(),
+                Action::Delete => KeyCode::Delete.into(),
                 Action::Edit => KeyCode::Char('e').into(),
                 Action::Reset => KeyCode::Char('z').into(),
                 Action::View => KeyCode::Char('v').into(),
@@ -194,7 +195,7 @@ mod tests {
     use super::*;
     use crossterm::event::KeyEventState;
     use rstest::rstest;
-    use slumber_core::assert_matches;
+    use slumber_util::assert_matches;
 
     /// Helper to create a key event
     fn key_event(
