@@ -1,7 +1,7 @@
 //! Async message passing! This is how inputs and other external events trigger
 //! state updates.
 
-use crate::view::Confirm;
+use crate::{util::TempFile, view::Confirm};
 use anyhow::Context;
 use derive_more::From;
 use mime::Mime;
@@ -12,10 +12,10 @@ use slumber_core::{
         BuildOptions, Exchange, RequestBuildError, RequestError, RequestId,
         RequestRecord,
     },
-    template::{Prompt, Prompter, Select, Template, TemplateChunk},
-    util::ResultTraced,
+    template::{Prompt, ResponseChannel, Select, Template, TemplateChunk},
 };
-use std::{fmt::Debug, path::PathBuf, sync::Arc};
+use slumber_util::ResultTraced;
+use std::{fmt::Debug, sync::Arc};
 use tokio::sync::mpsc::UnboundedSender;
 use tracing::trace;
 
@@ -40,25 +40,15 @@ impl MessageSender {
     }
 }
 
-/// Use the message stream to prompt the user for input when needed for a
-/// template. The message will be routed to the view so it can show the prompt,
-/// and the given returner will be used to send the submitted value back.
-impl Prompter for MessageSender {
-    fn prompt(&self, prompt: Prompt) {
-        self.send(Message::PromptStart(prompt));
-    }
-
-    fn select(&self, select: Select) {
-        self.send(Message::SelectStart(select));
-    }
-}
-
 /// A message triggers some *asynchronous* action. Most state modifications can
 /// be made synchronously by the input handler, but some require async handling
 /// at the top level. Messages can be triggered from anywhere (via the TUI
 /// context), but are all handled by the top-level controller.
 #[derive(derive_more::Debug)]
 pub enum Message {
+    /// Clear the terminal. Use this before deferring to a subprocess
+    ClearTerminal,
+
     /// Trigger collection reload
     CollectionStartReload,
     /// Store a reloaded collection value in state
@@ -79,36 +69,57 @@ pub enum Message {
     /// Copy some text to the clipboard
     CopyText(String),
 
+    /// Trigger a redraw. This should be called whenever we have reason to
+    /// believe the UI may have changed due to a background task
+    Draw,
+
     /// An error occurred in some async process and should be shown to the user
     Error { error: anyhow::Error },
 
     /// Open a file in the user's external editor
     FileEdit {
-        path: PathBuf,
-        /// Function to call once the edit is done. The original path will be
-        /// passed back
+        file: TempFile,
+        /// Function to call once the edit is done. The original file will be
+        /// passed back so the caller can read its contents before it gets
+        /// dropped+deleted.
         #[debug(skip)]
-        on_complete: Callback<PathBuf>,
+        on_complete: Callback<TempFile>,
     },
     /// Open a file to be viewed in the user's external pager
     FileView {
-        path: PathBuf,
+        file: TempFile,
         /// MIME type of the file being viewed
         mime: Option<Mime>,
     },
 
     /// Launch an HTTP request from the given recipe/profile.
     HttpBeginRequest,
+    /// Announce that we've started building an HTTP request that was triggered
+    /// by another request.
+    HttpBuildingTriggered {
+        id: RequestId,
+        profile_id: Option<ProfileId>,
+        recipe_id: RecipeId,
+    },
     /// Request failed to build
-    HttpBuildError { error: RequestBuildError },
+    HttpBuildError { error: Arc<RequestBuildError> },
     /// We launched the HTTP request
     HttpLoading { request: Arc<RequestRecord> },
     /// The HTTP request either succeeded or failed. We don't need to store the
     /// recipe ID here because it's in the inner container already. Combining
-    /// these two cases saves a bit of boilerplate.
-    HttpComplete(Result<Exchange, RequestError>),
+    /// these two cases saves a bit of boilerplate. The error must be wrapped
+    /// in `Arc` because it may need to be shared. Triggered requests need their
+    /// error returned to the template engine, but also need to be inserted into
+    /// the request store.
+    HttpComplete(Result<Exchange, Arc<RequestError>>),
     /// Cancel an HTTP request
     HttpCancel(RequestId),
+    /// Get the most recent _completed_ request for a recipe+profile combo
+    HttpGetLatest {
+        profile_id: Option<ProfileId>,
+        recipe_id: RecipeId,
+        channel: ResponseChannel<Option<Exchange>>,
+    },
 
     /// User input from the terminal
     Input {
@@ -154,10 +165,6 @@ pub enum Message {
         #[debug(skip)]
         on_complete: Callback<Vec<TemplateChunk>>,
     },
-
-    /// Trigger a redraw. This should be called whenever we have reason to
-    /// believe the UI may have changed due to a background task
-    Tick,
 }
 
 /// A static callback included in a message
