@@ -13,7 +13,7 @@ mod util;
 pub use common::modal::{IntoModal, ModalPriority};
 pub use context::{UpdateContext, ViewContext};
 pub use styles::Styles;
-pub use util::{Confirm, PreviewPrompter};
+pub use util::{Confirm, PreviewPrompter, TuiPrompter};
 
 use crate::{
     context::TuiContext,
@@ -22,17 +22,22 @@ use crate::{
     util::ResultReported,
     view::{
         common::modal::Modal,
-        component::{Component, Root},
+        component::{Component, Root, RootProps},
         debug::DebugMonitor,
+        draw::Generate,
         event::Event,
     },
 };
 use anyhow::anyhow;
-use ratatui::Frame;
+use ratatui::{
+    Frame,
+    layout::{Constraint, Layout},
+    text::Text,
+};
 use slumber_config::Action;
 use slumber_core::{
-    collection::{CollectionFile, ProfileId},
-    db::CollectionDatabase,
+    collection::{Collection, CollectionFile, ProfileId},
+    database::CollectionDatabase,
     http::RequestId,
 };
 use std::{fmt::Debug, sync::Arc};
@@ -59,16 +64,11 @@ pub struct View {
 
 impl View {
     pub fn new(
-        collection_file: &CollectionFile,
-        request_store: &RequestStore,
+        collection: &Arc<Collection>,
         database: CollectionDatabase,
         messages_tx: MessageSender,
     ) -> Self {
-        ViewContext::init(
-            Arc::clone(&collection_file.collection),
-            database,
-            messages_tx,
-        );
+        ViewContext::init(Arc::clone(collection), database, messages_tx);
 
         let debug_monitor = if TuiContext::get().config.debug {
             Some(DebugMonitor::default())
@@ -77,17 +77,25 @@ impl View {
         };
 
         Self {
-            root: Root::new(&collection_file.collection, request_store).into(),
+            root: Root::new(collection).into(),
             debug_monitor,
         }
     }
 
     /// Draw the view to screen. This needs access to the input engine in order
     /// to render input bindings as help messages to the user.
-    pub fn draw<'a>(&'a self, frame: &'a mut Frame) {
-        fn draw_impl(root: &Component<Root>, frame: &mut Frame) {
+    pub fn draw<'a>(
+        &'a self,
+        frame: &'a mut Frame,
+        request_store: &RequestStore,
+    ) {
+        fn draw_impl(
+            root: &Component<Root>,
+            frame: &mut Frame,
+            request_store: &RequestStore,
+        ) {
             let chunk = frame.area();
-            root.draw(frame, (), chunk, true);
+            root.draw(frame, RootProps { request_store }, chunk, true);
         }
 
         // If the screen is too small to render anything, don't try. This avoids
@@ -99,12 +107,43 @@ impl View {
 
         // If debug monitor is enabled, use it to capture the view duration
         if let Some(debug_monitor) = &self.debug_monitor {
-            debug_monitor.draw(frame, |frame| draw_impl(&self.root, frame));
+            debug_monitor.draw(frame, |frame| {
+                draw_impl(&self.root, frame, request_store);
+            });
         } else {
-            draw_impl(&self.root, frame);
+            draw_impl(&self.root, frame, request_store);
         }
     }
 
+    /// When the collection fails to load on first launch, we can't show the
+    /// full UI yet. This draws an error state. The TUI loop should be watching
+    /// the collection file so we can retry initialization when the error is
+    /// fixed.
+    pub fn draw_collection_load_error(
+        frame: &mut Frame,
+        collection_file: &CollectionFile,
+        error: &anyhow::Error,
+    ) {
+        let context = TuiContext::get();
+        let [message_area, _, error_area] = Layout::vertical([
+            Constraint::Length(2),
+            Constraint::Length(1), // A nice gap
+            Constraint::Min(1),
+        ])
+        .areas(frame.area());
+        frame.render_widget(error.generate(), error_area);
+        frame.render_widget(
+            Text::styled(
+                format!(
+                    "Watching {collection_file} for changes...\n{} to exit",
+                    context.input_engine.binding_display(Action::ForceQuit),
+                ),
+                context.styles.text.primary,
+            ),
+            message_area,
+        );
+    }
+
     /// ID of the selected profile. `None` iff the list is empty
     pub fn selected_profile_id(&self) -> Option<&ProfileId> {
         self.root.data().selected_profile_id()
@@ -128,20 +167,14 @@ impl View {
             .reported(&ViewContext::messages_tx());
     }
 
-    /// Notify the view that a request's state has changed in the store. If the
-    /// request is selected, view state will be updated accordingly
-    pub fn update_request(&mut self, request_state: &RequestState) {
-        self.root.data_mut().update_request(request_state);
-    }
-
     /// Queue an event to open a new modal. The input can be anything that
     /// converts to modal content
-    pub fn open_modal(&mut self, modal: impl IntoModal + 'static) {
+    pub fn open_modal(&self, modal: impl IntoModal + 'static) {
         modal.into_modal().open();
     }
 
     /// Queue an event to send an informational notification to the user
-    pub fn notify(&mut self, message: impl ToString) {
+    pub fn notify(&self, message: impl ToString) {
         ViewContext::notify(message);
     }
 
@@ -149,11 +182,11 @@ impl View {
     /// user. If possible, a bound action is provided which tells us what
     /// abstract action the input maps to.
     pub fn handle_input(
-        &mut self,
+        &self,
         event: crossterm::event::Event,
         action: Option<Action>,
     ) {
-        ViewContext::push_event(Event::Input { event, action })
+        ViewContext::push_event(Event::Input { event, action });
     }
 
     /// Drain all view events from the queue. The component three will process
@@ -192,7 +225,7 @@ impl View {
                 // directly convert it to anyhow
                 ViewContext::send_message(Message::Error {
                     error: anyhow!("Error copying text: {error}"),
-                })
+                });
             }
         }
     }
@@ -202,19 +235,18 @@ impl View {
 mod tests {
     use super::*;
     use crate::test_util::{
-        assert_events, harness, terminal, TestHarness, TestTerminal,
+        TestHarness, TestTerminal, assert_events, harness, terminal,
     };
     use rstest::rstest;
-    use slumber_core::{collection::Collection, test_util::Factory};
+    use slumber_core::collection::Collection;
+    use slumber_util::Factory;
 
     /// Test view handling and drawing during initial view setup
     #[rstest]
     fn test_initial_draw(harness: TestHarness, terminal: TestTerminal) {
         let collection = Collection::factory(());
-        let collection_file = CollectionFile::factory(collection);
         let mut view = View::new(
-            &collection_file,
-            &harness.request_store.borrow(),
+            &collection.into(),
             harness.database.clone(),
             harness.messages_tx().clone(),
         );
@@ -233,7 +265,7 @@ mod tests {
         assert_events!(Event::Emitted { .. }, Event::Emitted { .. },);
 
         // Nothing new
-        terminal.draw(|frame| view.draw(frame));
+        terminal.draw(|frame| view.draw(frame, &request_store));
         assert_events!(Event::Emitted { .. }, Event::Emitted { .. },);
 
         // *Now* the queue is drained
