@@ -1,7 +1,7 @@
 use crate::{
     context::TuiContext,
     message::Message,
-    view::{draw::Generate, state::Identified, util::highlight, ViewContext},
+    view::{ViewContext, draw::Generate, state::Identified, util::highlight},
 };
 use ratatui::{
     buffer::Buffer,
@@ -80,7 +80,7 @@ impl TemplatePreview {
                     &destination,
                     content_type,
                     style,
-                )
+                );
             };
 
             ViewContext::send_message(Message::TemplatePreview {
@@ -137,7 +137,7 @@ impl Generate for &TemplatePreview {
 
 impl Widget for &TemplatePreview {
     fn render(self, area: Rect, buf: &mut Buffer) {
-        (&**self.text()).render(area, buf)
+        (&**self.text()).render(area, buf);
     }
 }
 
@@ -179,7 +179,7 @@ impl TextStitcher {
     /// Add one chunk to the text. This will recursively split on any line
     /// breaks in the text until it reaches the end.
     fn add_chunk(&mut self, chunk_text: String, style: Style) {
-        let ends_in_newline = chunk_text.ends_with("\n");
+        let ends_in_newline = chunk_text.ends_with('\n');
 
         // The first line should extend the final line of the current text,
         // because there isn't necessarily a line break between chunks
@@ -214,7 +214,7 @@ impl TextStitcher {
     /// text out of the chunk, because it's all stashed behind Arcs
     fn get_chunk_text(chunk: &TemplateChunk) -> String {
         match chunk {
-            TemplateChunk::Raw(text) => text.deref().clone(),
+            TemplateChunk::Raw(text) => text.deref().into(),
             TemplateChunk::Rendered { value, sensitive } => {
                 if *sensitive {
                     // Hide sensitive values. Ratatui has a Masked type, but
@@ -238,15 +238,16 @@ impl TextStitcher {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::test_util::{harness, TestHarness};
+    use crate::test_util::{TestHarness, harness};
     use indexmap::indexmap;
     use pretty_assertions::assert_eq;
     use rstest::rstest;
     use slumber_core::{
         collection::{Chain, ChainSource, Collection, Profile},
         template::TemplateContext,
-        test_util::{by_id, invalid_utf8_chain, Factory},
+        test_util::{by_id, invalid_utf8_chain},
     };
+    use slumber_util::Factory;
 
     /// Test line breaks, multi-byte characters, and binary data
     #[rstest]
