@@ -13,6 +13,6 @@ mod response_view;
 mod root;
 
 pub use internal::Component;
-pub use root::Root;
+pub use root::{Root, RootProps};
 // Exported for the view context
 pub use recipe_pane::RecipeOverrideStore;
