@@ -1,6 +1,7 @@
 use crate::{
     util::ResultReported,
     view::{
+        ViewContext,
         common::{
             actions::{IntoMenuAction, MenuAction},
             modal::Modal,
@@ -8,23 +9,22 @@ use crate::{
             text_box::TextBox,
         },
         component::{
+            Component,
             misc::TextBoxModal,
             recipe_pane::persistence::{RecipeOverrideKey, RecipeTemplate},
-            Component,
         },
         context::UpdateContext,
         draw::{Draw, DrawMetadata, Generate},
         event::{Child, Emitter, Event, EventHandler, OptionEvent, ToEmitter},
         state::select::{SelectState, SelectStateEvent, SelectStateEventType},
         util::persistence::{Persisted, PersistedKey, PersistedLazy},
-        ViewContext,
     },
 };
 use itertools::Itertools;
 use ratatui::{
+    Frame,
     layout::Constraint,
     widgets::{Row, TableState},
-    Frame,
 };
 use slumber_config::Action;
 use slumber_core::{
@@ -162,7 +162,7 @@ where
                 // so we don't need to plumb the index/key through
                 RecipeTableMenuAction::Edit { .. } => self.edit_selected_row(),
                 RecipeTableMenuAction::Reset { .. } => {
-                    self.reset_selected_row()
+                    self.reset_selected_row();
                 }
             })
     }
@@ -261,7 +261,7 @@ where
     }
 }
 
-#[derive(Clone)]
+#[derive(Debug)]
 pub struct RecipeFieldTableProps<'a> {
     /// Label for the left column in the table
     pub key_header: &'a str,
@@ -294,7 +294,8 @@ struct RowState<K: PersistedKey<Value = bool>> {
 }
 
 impl<K: PersistedKey<Value = bool>> Generate for &RowState<K> {
-    type Output<'this> = Row<'this>
+    type Output<'this>
+        = Row<'this>
     where
         Self: 'this;
 
@@ -386,11 +387,11 @@ where
 mod tests {
     use super::*;
     use crate::{
-        test_util::{harness, terminal, TestHarness, TestTerminal},
+        test_util::{TestHarness, TestTerminal, harness, terminal},
         view::{
             component::{
-                recipe_pane::persistence::RecipeOverrideValue,
                 RecipeOverrideStore,
+                recipe_pane::persistence::RecipeOverrideValue,
             },
             test_util::TestComponent,
         },
@@ -399,7 +400,8 @@ mod tests {
     use persisted::PersistedStore;
     use rstest::rstest;
     use serde::Serialize;
-    use slumber_core::{collection::RecipeId, test_util::Factory};
+    use slumber_core::collection::RecipeId;
+    use slumber_util::Factory;
 
     #[derive(Debug, Serialize, persisted::PersistedKey)]
     #[persisted(Option<String>)]
@@ -436,15 +438,13 @@ mod tests {
                 },
             ),
         ];
+
         let mut component = TestComponent::builder(
             &harness,
             &terminal,
             RecipeFieldTable::new("Row", TestRowKey(recipe_id.clone()), rows),
         )
-        .with_props(RecipeFieldTableProps {
-            key_header: "Key",
-            value_header: "Value",
-        })
+        .with_props(props_factory())
         .build();
 
         // Check initial state
@@ -455,7 +455,7 @@ mod tests {
 
         // Disable the second row
         component
-            .int()
+            .int_props(props_factory)
             .send_keys([KeyCode::Down, KeyCode::Char(' ')])
             .assert_empty();
         let selected_row = component.data().select.data().selected().unwrap();
@@ -467,7 +467,10 @@ mod tests {
         );
 
         // Re-enable the row
-        component.int().send_key(KeyCode::Char(' ')).assert_empty();
+        component
+            .int_props(props_factory)
+            .send_key(KeyCode::Char(' '))
+            .assert_empty();
         let selected_row = component.data().select.data().selected().unwrap();
         assert!(*selected_row.enabled);
         assert_eq!(
@@ -500,15 +503,13 @@ mod tests {
                 },
             ),
         ];
+
         let mut component = TestComponent::builder(
             &harness,
             &terminal,
             RecipeFieldTable::new("Row", TestRowKey(recipe_id.clone()), rows),
         )
-        .with_props(RecipeFieldTableProps {
-            key_header: "Key",
-            value_header: "Value",
-        })
+        .with_props(props_factory())
         .build();
 
         // Check initial state
@@ -519,7 +520,7 @@ mod tests {
 
         // Edit the second row
         component
-            .int()
+            .int_props(props_factory)
             // Open the modal
             .send_keys([KeyCode::Down, KeyCode::Char('e')])
             .send_text("!!!")
@@ -538,7 +539,10 @@ mod tests {
         );
 
         // Reset edited state
-        component.int().send_key(KeyCode::Char('z')).assert_empty();
+        component
+            .int_props(props_factory)
+            .send_key(KeyCode::Char('z'))
+            .assert_empty();
         let selected_row = component.data().select.data().selected().unwrap();
         assert!(!selected_row.value.is_overridden());
     }
@@ -556,19 +560,17 @@ mod tests {
                 key: "row0".into(),
             },
         )];
+
         let mut component = TestComponent::builder(
             &harness,
             &terminal,
             RecipeFieldTable::new("Row", TestRowKey(recipe_id.clone()), rows),
         )
-        .with_props(RecipeFieldTableProps {
-            key_header: "Key",
-            value_header: "Value",
-        })
+        .with_props(props_factory())
         .build();
 
         component
-            .int()
+            .int_props(props_factory)
             .open_actions()
             .send_keys([KeyCode::Enter, KeyCode::Char('!'), KeyCode::Enter])
             .assert_empty();
@@ -614,10 +616,7 @@ mod tests {
             &terminal,
             RecipeFieldTable::new("Row", TestRowKey(recipe_id.clone()), rows),
         )
-        .with_props(RecipeFieldTableProps {
-            key_header: "Key",
-            value_header: "Value",
-        })
+        .with_props(props_factory())
         .build();
 
         assert_eq!(
@@ -630,4 +629,11 @@ mod tests {
             .collect(),
         );
     }
+
+    fn props_factory() -> RecipeFieldTableProps<'static> {
+        RecipeFieldTableProps {
+            key_header: "Key",
+            value_header: "Value",
+        }
+    }
 }
