@@ -7,7 +7,7 @@ use crate::{
         state::Notification,
     },
 };
-use slumber_core::{collection::Collection, db::CollectionDatabase};
+use slumber_core::{collection::Collection, database::CollectionDatabase};
 use std::{cell::RefCell, sync::Arc};
 use tracing::debug;
 
@@ -70,8 +70,8 @@ impl ViewContext {
                 recipe_override_store: Default::default(),
                 event_queue: EventQueue::default(),
                 messages_tx,
-            })
-        })
+            });
+        });
     }
 
     /// Execute a function with read-only access to the context
@@ -153,7 +153,7 @@ impl ViewContext {
         Self::with(|context| {
             let refs: Vec<_> = context.event_queue.to_vec();
             f(refs.as_slice());
-        })
+        });
     }
 }
 
@@ -166,9 +166,9 @@ pub struct UpdateContext<'a> {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::test_util::{assert_events, harness, TestHarness};
+    use crate::test_util::{TestHarness, assert_events, harness};
     use rstest::rstest;
-    use slumber_core::assert_matches;
+    use slumber_util::assert_matches;
 
     #[rstest]
     fn test_event_queue(_harness: TestHarness) {
